package checkers.nonnull;

import java.lang.annotation.Annotation;

import checkers.quals.*;
import checkers.types.*;

/**
 * Represents a class type that may be qualified with {@code @NonNull} as the
 * result of a declaration or use with {@code @NonNull} type annotations. It
 * additionally permits the use of a {@code @Nullable} type annotation, which
 * specifies that a variable may become null.
 *
 * @todo this class should be updated to use {@link AnnotatedClassType#exclude}
 */
public class NonnullAnnotatedClassType extends AnnotatedClassType {

    @Override
    public boolean hasAnnotationAt(Class<? extends Annotation> annotation, AnnotationLocation location) {

        // Override to false if Nullable is present.
        if (annotation == NonNull.class && super.hasAnnotationAt(Nullable.class, location))
            return false;

        return super.hasAnnotationAt(annotation, location);
    }

}
