package checkers.quals;

import java.lang.annotation.Documented;
import java.lang.annotation.Retention;
import java.lang.annotation.RetentionPolicy;
import java.lang.annotation.Target;
import static java.lang.annotation.ElementType.*;

/**
 * Indicates that the mutability of the return value on a method depends on
 * the mutability of the arguments used at invocation. There are two possible
 * syntax for this annotation.
 *
 * If {@code @RoMaybe} is declared on a method return type without parameters,
 * the method return type will be readonly if and only if all of its parameters
 * annotated with {@code @RoMaybe} received arguments which were readonly.
 * For example:
 * <pre>
 *  &#064;RoMaybe aTestMethod(String a,
                              &#064;RoMaybe Object b,
                              &#064;RoMaybe Date c)
 * </pre>
 * has a readonly return type if and only if b and c are readonly.
 *
 * On the second possible usage, a array of strings is passed as a
 * parameter at each annotation. If a given argument is readonly, the
 * strings at its parameter declaration are "collected". If and only
 * if all the strings at the return type array are collected, the
 * return type is readonly. For example:
 * <pre>
 *  &#064;RoMaybe({"s1","s2"}) aTestMethod(String a,
                              &#064;RoMaybe({"s1","s2"}) Object b,
                              &#064;RoMaybe({"s1"}) Date c,
                              &#064;RoMaybe({"s2"}) Date d)
 * </pre>
 * has a readonly return type if the argument passed as b is readonly,
 * or if both arguments passed as c and d are readonly.
 *
 * WARNING: the syntax for this annotation is still experimental; expect
 * changes in future releases.
 */
@Documented
@Retention(RetentionPolicy.RUNTIME)
@Target({METHOD, PARAMETER})
public @interface RoMaybe {
    String[] value() default {};
}
